-- Daily Global Check - Professions plugin
-- Jadya EU-Well of Eternity
local addonName, addonTable = ...

local low, sfind, sgsub, tinsert = string.lower, string.find, string.gsub, table.insert

local prof_names = {}

DGC_ProfSaves = {}

local pID
local plugintitle = "Professions"
local pluginicon = "Interface\\Icons\\Trade_Alchemy"
local pTable
local profession_cd_ids = {}

-- seal of tempered fate weekly quests
local seal_quests = {36055,36058,36060,37452,37453,36057,37458,37459,36054,37454,37455,36056,37456,37457}

local function get_cache_reset(pid)
 if DGC_ProfSaves[pid] then 
  return DGC_ProfSaves[pid].garrison_cache_reset
 end
end

local function calculate_garrison_cache_quantity(pid)
 local t = get_cache_reset(pid)
 
 if not t or t == 0 then return "|cffFF0000"..GM_SURVEY_NOT_APPLICABLE.."|r" end
 
 local result = math.floor((time() - t) / 600)
 if result >= 500 then 
  result = "500 |cffFF0000("..LOC_TYPE_FULL..")|r"
 else
  result = result .. " / 500"
 end
 return result
end

local function getMaxNodes(s)
 if s == "garrison_herbs" then
  local bID = C_Garrison.GetOwnedBuildingInfo(63)
  return (bID == 137 and 15 or -- herb garden level 3
         (bID == 136 and 10 or -- herb garden level 2
         (bID == 29 and 7 or   -- herb garden level 1
		 0)))
 elseif s == "garrison_mines" then
  local bID = C_Garrison.GetOwnedBuildingInfo(59)
  return (bID == 63 and 18 or -- mine level 3
         (bID == 62 and 13 or -- mine level 2
         (bID == 61 and 8 or  -- mine level 1
         0)))
 else 
  return 0
 end
end

local function isquestcompleted(questID)
 if type(questID) == "number" then
  return IsQuestFlaggedCompleted(questID)
 elseif questID == "seal_of_tempered_fate" then
  local counter = 0
  for _,v in pairs(seal_quests) do
   if IsQuestFlaggedCompleted(v) then counter = counter + 1 end
   if counter == 3 then break end
  end
  return true, counter.." / 3"
 elseif questID == "garrison_cache" then
  local quantity = calculate_garrison_cache_quantity(DailyGlobalCheck.selectedpID)
  return true, quantity
 elseif questID == "garrison_herbs" or questID == "garrison_mines" then
  local data = DGC_ProfSaves[pID]
  local maxnodes = getMaxNodes(questID)
  if maxnodes > 0 then
   local result
   if not data or not data[questID] then 
    result = 0
   elseif time() > (data[questID].reset or 0) then
    data[questID] = nil
    result = 0
   else
    result = data[questID].count
   end
   return true, result.." / "..maxnodes
  end
 elseif questID == "weekly_follower" then
  local data = DGC_ProfSaves[pID]
  if not data then return end
  if not data.weekly_follower or time() > data.weekly_follower.reset then
   data.weekly_follower = nil
  else
   return true
  end
 elseif questID == "profession1" and profession_cd_ids[1] then
  local result = GetSpellCooldown(profession_cd_ids[1])
  return result > 0, prof_names[1]
 elseif questID == "profession2" and profession_cd_ids[2] then
  local result = GetSpellCooldown(profession_cd_ids[2])
  return result > 0, prof_names[2]
 end
end

-- this function is fired just before dgc draws the quests list
-- it sets the right cache quantity for an alt in its dgc savedvariables
local function onshow()
 if DailyGlobalCheck.selectedpID == pID then return end

 local data = DailyGlobalCheck_CharData[DailyGlobalCheck.selectedpID] and DailyGlobalCheck_CharData[DailyGlobalCheck.selectedpID].data
 
 if not data then return end
 
 local t = type(data["garrison_cache"])

 if t == "table" then -- DGC old character save format
  data["garrison_cache"][2] = calculate_garrison_cache_quantity(DailyGlobalCheck.selectedpID)
 else -- DGC new character save format
  data["garrison_cache"] = calculate_garrison_cache_quantity(DailyGlobalCheck.selectedpID).."~"
 end
end

local list = {
 ["Title"] = plugintitle,
 ["Icon"]  = pluginicon,
 ["Version"] = 1003,
 ["Data"] = {}, -- empty, for compatibility reasons
 ["Order"] = {
              { -- page 1
               {GARRISON_LOCATION_TOOLTIP,"garrison_cache","seal_of_tempered_fate","weekly_follower"},
			   {"","garrison_herbs","garrison_mines"},
			   {BATTLE_PET_SOURCE_4,"profession1","profession2",--[[37319]]},
			  },
             },
 ["Overrides"] = {["isquestcompleted"] = isquestcompleted},
 ["OnShow"] = onshow,
 MultiCharsEnabled = true,
}

local questsdata =  {["garrison_cache"] = {GARRISON_LOCATION_TOOLTIP,GARRISON_CACHE,"","",nil,nil,"Q"},
                     ["seal_of_tempered_fate"] = {GARRISON_LOCATION_TOOLTIP,"Seal of Tempered Fate","","",nil,nil,"W"},
					 ["weekly_follower"] = {GARRISON_LOCATION_TOOLTIP,"Weekly Follower","","",nil,nil,"W"},
                     ["profession1"] = {"",BATTLE_PET_SOURCE_4..1,"",""},
			         ["profession2"] = {"",BATTLE_PET_SOURCE_4..2,"",""},
					 ["garrison_herbs"] = {GARRISON_LOCATION_TOOLTIP,"Garrison Herbs"},
					 ["garrison_mines"] = {GARRISON_LOCATION_TOOLTIP,"Garrison Mines"},
			         [37319] = {GARRISON_LOCATION_TOOLTIP,"Gem Boutique Daily","",""},
					} -- 37320 37321 37323 37324 37325

local function Initialize()
 DailyGlobalCheck:LoadPlugin(list, questsdata)
 pTable = DailyGlobalCheck_CharData
end

local function setWeeklyFollowerTrue()
 local calcResets = DailyGlobalCheck.CharFunctions.CalculateResets
 if not calcResets then return end
 
 if not DGC_ProfSaves[pID] then
  DGC_ProfSaves[pID] = {}
 end

 local wr = calcResets()
 DGC_ProfSaves[pID].weekly_follower = {reset = wr}
end

local function checkLoot(s)
 local function CounterPlusPlus(key)
  if not DGC_ProfSaves[pID] then
   DGC_ProfSaves[pID] = {}
  end
  
  local data = DGC_ProfSaves[pID]
  local now = time()

  if not data[key] or data[key].reset < now then
   data[key] = {}
  end
  
  if not data[key].count then
   -- I need to reset them manually since they're saved in the professions savedvariables
   data[key].reset = now + GetQuestResetTime()
   data[key].count = 1
  else
   data[key].count = data[key].count + 1
  end
 end
 
 local LOOT_SELF_REGEX = gsub(LOOT_ITEM_SELF, "%%s", "(.+)")
 local _, _, itemlink = s:find(LOOT_SELF_REGEX)
 if not itemlink then return end
 local _, _, itemID = itemlink:find("item:(%d+):")
 
 if itemID == "116053" then
  CounterPlusPlus("garrison_herbs")
 elseif itemID == "115508" then
  CounterPlusPlus("garrison_mines")
 end
end

local initialized
local eventframe = CreateFrame("Frame")
eventframe:RegisterEvent("ADDON_LOADED")
eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
local function eventhandler(self, event, ...)
 if event == "ADDON_LOADED" then
  if not DailyGlobalCheck then
   return
  elseif ... == addonName then
   local pName = GetUnitName("player")
   local pRealm = GetRealmName()
   pID = pName.."-"..pRealm
   Initialize()
   initialized = true
   eventframe:RegisterEvent("SHOW_LOOT_TOAST")
   eventframe:RegisterEvent("CHAT_MSG_LOOT")
  elseif ... == "Blizzard_GarrisonUI" then
   GarrisonRecruitSelectFrame.FollowerSelection.Recruit1.HireRecruits:HookScript("OnClick", setWeeklyFollowerTrue)
   GarrisonRecruitSelectFrame.FollowerSelection.Recruit2.HireRecruits:HookScript("OnClick", setWeeklyFollowerTrue)
   GarrisonRecruitSelectFrame.FollowerSelection.Recruit3.HireRecruits:HookScript("OnClick", setWeeklyFollowerTrue)
  end
 elseif event == "CHAT_MSG_LOOT" then
  if not C_Garrison.IsOnGarrisonMap() then return end
  checkLoot(...)
 elseif event == "PLAYER_ENTERING_WORLD" then
  eventframe:UnregisterEvent("PLAYER_ENTERING_WORLD")
  if DailyGlobalCheck and not initialized then Initialize() end
  
  -- seal of fate localization
  local Sname, _, Sicon  = GetCurrencyInfo(994)
  questsdata["seal_of_tempered_fate"][2] = Sname
  questsdata["seal_of_tempered_fate"][3] = "|T"..Sicon..":12|t"

  local p1, p2 = GetProfessions()

   local function addProfession(p, index)
	if p == GetSpellInfo(158758) then -- tailoring
	 profession_cd_ids[index] = 168835
	 prof_names[index] = GetSpellInfo(168835)
	elseif p == GetSpellInfo(158716) then -- enchanting
	 profession_cd_ids[index] = 169092
	 prof_names[index] = GetSpellInfo(169092)
	elseif p == GetSpellInfo(156606) then -- alchemy
	 profession_cd_ids[index] = 156587
	 prof_names[index] = GetSpellInfo(156587)
	elseif p == GetSpellInfo(158748) then -- inscription
	 profession_cd_ids[index] = 169081
	 prof_names[index] = GetSpellInfo(169081)
	elseif p == GetSpellInfo(110403) then -- engineering
	 profession_cd_ids[index] = 169080
	 prof_names[index] = GetSpellInfo(169080)
    elseif p == GetSpellInfo(110423) then -- leatherworking
	 profession_cd_ids[index] = 171391
	 prof_names[index] = GetSpellInfo(171391)
    elseif p == GetSpellInfo(158737) then -- blacksmithing
	 profession_cd_ids[index] = 171690
	 prof_names[index] = GetSpellInfo(171690)
    elseif p == GetSpellInfo(158750) then -- jewelcrafting
	 profession_cd_ids[index] = 170700
	 prof_names[index] = GetSpellInfo(170700)
    else
     profession_cd_ids[index] = -1
     prof_names[index] = NONE
	end
   end
  
  local pname
  if p1 then
   pname = GetProfessionInfo(p1)
   addProfession(pname, 1)
  else
   profession_cd_ids[1] = -1
   prof_names[1] = NONE
  end
  if p2 then
   pname = GetProfessionInfo(p2)
   addProfession(pname, 2)
  else
   profession_cd_ids[2] = -1
   prof_names[2] = NONE
  end
 elseif event == "SHOW_LOOT_TOAST" then
  local z = GetZoneText()
  -- not in garrison
  if z ~= GetMapNameByID(971) and z ~= GetMapNameByID(976) then
   return
  end
   
  local typeIdentifier, itemLink, quantity, specID, sex, isPersonal, lootSource = ...
  
  if not typeIdentifier or low(typeIdentifier) ~= low(CURRENCY) then return end

  local _, _, _, _, ID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*):?(%-?%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
  
   if not DGC_ProfSaves[pID] then
    DGC_ProfSaves[pID] = {}
   end
  
  if lootSource == 10 and ID == "824" then
   DGC_ProfSaves[pID].garrison_cache_reset = time()
  end
 end
end
eventframe:SetScript("OnEvent", eventhandler)